@@ -50,11 +50,11 @@ from datalad.tests.utils import (
     assert_status,
     create_tree,
     DEFAULT_BRANCH,
+    DEFAULT_REMOTE,
     eq_,
     get_datasets_topdir,
     integration,
     known_failure,
-    known_failure_appveyor,
     known_failure_githubci_win,
     known_failure_windows,
     neq_,
@@ -141,7 +141,6 @@ def test_clone_crcns(tdir, ds_path):
     assert_in(crcns.path, ds.subdatasets(result_xfm='paths'))
 
 
-@known_failure_appveyor
 @integration
 @skip_if_no_network
 @use_cassette('test_install_crcns')
@@ -262,7 +261,7 @@ def test_clone_isnot_recursive(path_src, path_nr, path_r):
 
     ds = clone(src, path_nr, result_xfm='datasets', return_type='item-or-list')
     ok_(ds.is_installed())
-    # check nothin is unintentionally installed
+    # check nothing is unintentionally installed
     subdss = ds.subdatasets(recursive=True)
     assert_result_count(subdss, len(subdss), state='absent')
     # this also means, subdatasets to be listed as not fulfilled:
@@ -276,7 +275,9 @@ def test_clone_into_dataset(source_path, top_path):
     source = Dataset(source_path).create()
     ds = create(top_path)
     assert_repo_status(ds.path)
-
+    # Note, we test against the produced history in DEFAULT_BRANCH, not what it
+    # turns into in an adjusted branch!
+    hexsha_before = ds.repo.get_hexsha(DEFAULT_BRANCH)
     subds = ds.clone(source, "sub",
                      result_xfm='datasets', return_type='item-or-list')
     ok_((subds.pathobj / '.git').is_dir())
@@ -290,6 +291,13 @@ def test_clone_into_dataset(source_path, top_path):
     sds = ds.subdatasets("sub")
     assert_result_count(sds, 1, action='subdataset')
     eq_(sds[0]['gitmodule_datalad-url'], source.path)
+    # Clone produced one commit including the addition to .gitmodule:
+    commits = list(ds.repo.get_branch_commits_(
+        branch=DEFAULT_BRANCH,
+        stop=hexsha_before
+    ))
+    assert_not_in(hexsha_before, commits)
+    eq_(len(commits), 1)
 
     # but we could also save while installing and there should be no side-effect
     # of saving any other changes if we state to not auto-save changes
@@ -464,7 +472,7 @@ def test_clone_isnt_a_smartass(origin_path, path):
     with chpwd(path):
         # no were are inside a dataset clone, and we make another one
         # we do not want automatic subdatasetification without given a dataset
-        # explicitely
+        # explicitly
         clonedsub = clone(origin, 'testsub',
                           result_xfm='datasets', return_type='item-or-list')
     # correct destination
@@ -499,8 +507,6 @@ def test_clone_report_permission_issue(tdir):
         )
 
 
-# Started to hang on appveyor.
-@known_failure_appveyor  #FIXME - hangs
 @skip_if_no_network
 @with_tempfile
 def test_autoenabled_remote_msg(path):
@@ -531,24 +537,41 @@ def test_clone_autoenable_msg_handles_sameas(repo, clone_path):
 
 
 def test_installationpath_from_url():
-    cases = (
+    # cases for all OSes
+    cases = [
         'http://example.com/lastbit',
         'http://example.com/lastbit.git',
         'http://lastbit:8000',
-    ) + (
+        # SSH
+        'hostname:lastbit',
+        'hostname:lastbit/',
+        'hostname:subd/lastbit',
+        'hostname:/full/path/lastbit',
+        'hostname:lastbit/.git',
+        'hostname:lastbit/.git/',
+        'hostname:/full/path/lastbit/.git',
+        'full.hostname.com:lastbit/.git',
+        'user@full.hostname.com:lastbit/.git',
+        'ssh://user:passw@full.hostname.com/full/path/lastbit',
+        'ssh://user:passw@full.hostname.com/full/path/lastbit/',
+        'ssh://user:passw@full.hostname.com/full/path/lastbit/.git',
+    ]
+    # OS specific cases
+    cases += [
         'C:\\Users\\mih\\AppData\\Local\\Temp\\lastbit',
         'C:\\Users\\mih\\AppData\\Local\\Temp\\lastbit\\',
         'Temp\\lastbit',
         'Temp\\lastbit\\',
         'lastbit.git',
         'lastbit.git\\',
-    ) if on_windows else (
+    ] if on_windows else [
         'lastbit',
         'lastbit/',
         '/lastbit',
         'lastbit.git',
         'lastbit.git/',
-    )
+    ]
+
     for p in cases:
         eq_(_get_installationpath_from_url(p), 'lastbit')
     # we need to deal with quoted urls
@@ -621,7 +644,7 @@ def test_cfg_originorigin(path):
     eq_(
         Path(clone_lev2.siblings(
             'query',
-            name='origin-2',
+            name=DEFAULT_REMOTE + '-2',
             return_type='item-or-list')['url']),
         origin.pathobj
     )
@@ -632,8 +655,7 @@ def test_cfg_originorigin(path):
     with chpwd(path), swallow_logs(new_level=logging.DEBUG) as cml:
         clone_lev3 = clone('clone_lev2', 'clone_lev3')
         # we called git-annex-init; see gh-4367:
-        cml.assert_logged(msg=r"[^[]*Async run:\n cwd=.*\n"
-                              r" cmd=\[('git',.*'annex'|'git-annex'), 'init'",
+        cml.assert_logged(msg=r"[^[]*Run \[('git',.*'annex'|'git-annex'), 'init'",
                           match=False,
                           level='DEBUG')
     assert_result_count(
@@ -800,15 +822,15 @@ def test_ria_http(lcl, storepath, url):
         if not ds.repo.is_managed_branch():
             # test logic cannot handle adjusted branches
             eq_(origds.repo.get_hexsha(), cloneds.repo.get_hexsha())
-        ok_(cloneds.config.get('remote.origin.url').startswith(url))
-        eq_(cloneds.config.get('remote.origin.annex-ignore'), 'true')
+        ok_(cloneds.config.get(f'remote.{DEFAULT_REMOTE}.url').startswith(url))
+        eq_(cloneds.config.get(f'remote.{DEFAULT_REMOTE}.annex-ignore'), 'true')
         eq_(cloneds.config.get('datalad.get.subdataset-source-candidate-200origin'),
             'ria+%s#{id}' % url)
 
     # now advance the source dataset
     (ds.pathobj / 'newfile.txt').write_text('new')
     ds.save()
-    ds.publish(to='store')
+    ds.push(to='store')
     Runner(cwd=storeds_loc).run(['git', 'update-server-info'])
     # re-clone as before
     riaclone2 = clone(
@@ -850,10 +872,12 @@ def test_ria_http(lcl, storepath, url):
     if not ds.repo.is_managed_branch():
         # test logic cannot handle adjusted branches
         eq_(origds.repo.get_hexsha(), cloned_by_label.repo.get_hexsha())
-    ok_(cloned_by_label.config.get('remote.origin.url').startswith(url))
-    eq_(cloned_by_label.config.get('remote.origin.annex-ignore'), 'true')
+    ok_(cloned_by_label.config.get(
+        f'remote.{DEFAULT_REMOTE}.url').startswith(url))
+    eq_(cloned_by_label.config.get(f'remote.{DEFAULT_REMOTE}.annex-ignore'),
+        'true')
     # ... the clone candidates go with the label-based URL such that
-    # future get() requests acknowlege a (system-wide) configuration
+    # future get() requests acknowledge a (system-wide) configuration
     # update
     eq_(cloned_by_label.config.get('datalad.get.subdataset-source-candidate-200origin'),
         'ria+ssh://somelabel#{id}')
@@ -895,7 +919,7 @@ def _test_ria_postclonecfg(url, dsid, clone_path, superds):
     assert_result_count(res, 1,
                         status='ok',
                         path=str(riaclone.pathobj / 'test.txt'),
-                        message="from {}...".format("origin"
+                        message="from {}...".format(DEFAULT_REMOTE
                                                     if url.startswith('http')
                                                     else "store-storage"))
 
@@ -910,7 +934,7 @@ def _test_ria_postclonecfg(url, dsid, clone_path, superds):
 
     # publication dependency was set for store-storage but not for
     # anotherstore-storage:
-    eq_(riaclone.config.get("remote.origin.datalad-publish-depends",
+    eq_(riaclone.config.get(f"remote.{DEFAULT_REMOTE}.datalad-publish-depends",
                             get_all=True),
         "store-storage")
 
@@ -928,13 +952,13 @@ def _test_ria_postclonecfg(url, dsid, clone_path, superds):
     assert_result_count(res, 1,
                         status='ok',
                         path=str(riaclonesub.pathobj / 'testsub.txt'),
-                        message="from {}...".format("origin"
+                        message="from {}...".format(DEFAULT_REMOTE
                                                     if url.startswith('http')
                                                     else "store-storage"))
 
     # publication dependency was set for store-storage but not for
     # anotherstore-storage:
-    eq_(riaclonesub.config.get("remote.origin.datalad-publish-depends",
+    eq_(riaclonesub.config.get(f"remote.{DEFAULT_REMOTE}.datalad-publish-depends",
                                get_all=True),
         "store-storage")
 
@@ -948,7 +972,7 @@ def _test_ria_postclonecfg(url, dsid, clone_path, superds):
     assert_result_count(res, 2)
     # no ORA remote, no publication dependency:
     riaclonesubgit = Dataset(riaclone.pathobj / 'subdir' / 'subgit')
-    eq_(riaclonesubgit.config.get("remote.origin.datalad-publish-depends",
+    eq_(riaclonesubgit.config.get(f"remote.{DEFAULT_REMOTE}.datalad-publish-depends",
                                   get_all=True),
         None)
 
@@ -986,7 +1010,7 @@ def _test_ria_postclonecfg(url, dsid, clone_path, superds):
     assert_result_count(res, 1,
                         status='ok',
                         path=str(subds.pathobj / 'test.txt'),
-                        message="from {}...".format("origin"
+                        message="from {}...".format(DEFAULT_REMOTE
                                                     if url.startswith('http')
                                                     else "store-storage"))
 
@@ -1095,7 +1119,6 @@ def test_ria_postclonecfg():
         raise SkipTest("Can't create symlinks")
 
     from datalad.utils import make_tempfile
-    from datalad.tests.utils import HTTPPath
 
     with make_tempfile(mkdir=True) as lcl, make_tempfile(mkdir=True) as store, \
             make_tempfile(mkdir=True) as store2:
@@ -1118,6 +1141,61 @@ def test_ria_postclonecfg():
             "ssh://datalad-test:{}".format(Path(store).as_posix()), id
 
 
+@known_failure_windows
+@skip_ssh
+@with_tree(tree={'somefile.txt': 'some content'})
+@with_tempfile
+@with_tempfile
+def test_no_ria_postclonecfg(dspath, storepath, clonepath):
+
+    dspath = Path(dspath)
+    storepath = Path(storepath)
+    clonepath = Path(clonepath)
+
+    # Test that particular configuration(s) do NOT lead to a reconfiguration
+    # upon clone. (See gh-5628)
+
+    from datalad.customremotes.ria_utils import (
+        create_store,
+    )
+    from datalad.distributed.ora_remote import (
+        LocalIO,
+    )
+
+    ds = Dataset(dspath).create(force=True)
+    ds.save()
+    assert_repo_status(ds.path)
+
+    io = LocalIO()
+    create_store(io, storepath, '1')
+    file_url = "ria+{}".format(get_local_file_url(str(storepath)))
+    ssh_url = "ria+ssh://datalad-test:{}".format(storepath.as_posix())
+    ds.create_sibling_ria(file_url, "teststore",
+                          push_url=ssh_url, alias="testds")
+    ds.push('.', to='teststore')
+
+    # Now clone via SSH. Should not reconfigure although `url` doesn't match the
+    # URL we cloned from. However, `push-url` does.
+    riaclone = clone('{}#{}'.format(ssh_url, ds.id), clonepath)
+
+    # ORA remote is enabled (since URL still valid) but not reconfigured:
+    untouched_remote = riaclone.siblings(name='teststore-storage',
+                                         return_type='item-or-list')
+    assert_not_is_instance(untouched_remote, list)
+    ora_cfg = riaclone.repo.get_special_remotes()[
+        untouched_remote['annex-uuid']]
+    ok_(ora_cfg['url'] == file_url)
+    ok_(ora_cfg['push-url'] == ssh_url)
+
+    # publication dependency was still set (and it's the only one that was set):
+    eq_(riaclone.config.get(f"remote.{DEFAULT_REMOTE}.datalad-publish-depends",
+                            get_all=True),
+        "teststore-storage")
+
+    # we can still get the content
+    ds.get("somefile.txt")
+
+
 # fatal: Could not read from remote repository.
 @known_failure_githubci_win  # in datalad/git-annex as e.g. of 20201218
 @with_tempfile(mkdir=True)
@@ -1279,7 +1357,8 @@ def test_ephemeral(origin_path, bare_path,
                       where="local")
     # Note, that the only thing to test is git-annex-dead here,
     # if we couldn't symlink:
-    clone1.publish(to='origin', transfer_data='none' if can_symlink else 'auto')
+    clone1.push(to=DEFAULT_REMOTE,
+                   data='nothing' if can_symlink else 'auto')
     if not origin.repo.is_managed_branch():
         # test logic cannot handle adjusted branches
         eq_(origin.repo.get_hexsha(), clone1.repo.get_hexsha())
@@ -1460,8 +1539,8 @@ def test_fetch_git_special_remote(url_path, url, path):
     ds_special.save()
     ds_special.repo.call_git(["update-server-info"])
 
-    ds_a.repo.fetch("origin")
-    ds_a.repo.merge("origin/" + DEFAULT_BRANCH)
+    ds_a.repo.fetch(DEFAULT_REMOTE)
+    ds_a.repo.merge(f"{DEFAULT_REMOTE}/{DEFAULT_BRANCH}")
 
     ds_b = clone(ds_a.path, path / "other")
     ds_b.get("f1")
@@ -1500,3 +1579,85 @@ def test_clone_recorded_subds_reset(path):
     eq_(ds_b.subdatasets()[0]["gitshasum"],
         sub_repo.get_hexsha(
             sub_repo.get_corresponding_branch(branch) or branch))
+
+
+@with_tempfile
+@with_tempfile
+def test_clone_url_mapping(src_path, dest_path):
+    src = create(src_path)
+    dest = Dataset(dest_path)
+    # check that the impossible doesn't work
+    assert_raises(IncompleteResultsError, clone, 'rambo', dest_path)
+    # rather than adding test URL mapping here, consider
+    # test_url_mapping_specs(), it is cheaper there
+  
+    # anticipate windows test paths and escape them
+    escaped_subst = (r',rambo,%s' % src_path).replace('\\', '\\\\')
+    for specs in (
+            # we can clone with a simple substitution
+            {'datalad.clone.url-substitute.mike': escaped_subst},
+            # a prior match to a dysfunctional URL doesn't impact success
+            {
+                'datalad.clone.url-substitute.no': ',rambo,picknick',
+                'datalad.clone.url-substitute.mike': escaped_subst,
+            }):
+        try:
+            with patch.dict(dest.config._merged_store, specs):
+                clone('rambo', dest_path)
+        finally:
+            dest.remove(check=False)
+
+    # check submodule config impact
+    dest.create()
+    with patch.dict(dest.config._merged_store,
+                    {'datalad.clone.url-substitute.mike': escaped_subst}):
+        dest.clone('rambo', 'subds')
+    submod_rec = dest.repo.get_submodules()[0]
+    # we record the original-original URL
+    eq_(submod_rec['gitmodule_datalad-url'], 'rambo')
+    # and put the effective one as the primary URL
+    eq_(submod_rec['gitmodule_url'], src_path)
+
+
+_nomatch_map = {
+    'datalad.clone.url-substitute.nomatch': (
+        ',nomatch,NULL',
+    )
+}
+_windows_map = {
+    'datalad.clone.url-substitute.win': (
+        r',C:\\Users\\datalad\\from,D:\\to',
+    )
+}
+
+
+def test_url_mapping_specs():
+    from datalad.core.distributed.clone import _map_urls
+    cfg = ConfigManager()
+    for m, i, o in (
+            # path redirect on windows
+            (_windows_map,
+             r'C:\Users\datalad\from',
+             r'D:\to'),
+            # test standard github mapping, no pathc needed
+            ({},
+             'https://github.com/datalad/testrepo_gh/sub _1',
+             'https://github.com/datalad/testrepo_gh-sub__1'),
+            # and on deep subdataset too
+            ({},
+             'https://github.com/datalad/testrepo_gh/sub _1/d/sub_-  1',
+             'https://github.com/datalad/testrepo_gh-sub__1-d-sub_-_1'),
+            # test that the presence of another mapping spec doesn't ruin
+            # the outcome
+            (_nomatch_map,
+             'https://github.com/datalad/testrepo_gh/sub _1',
+             'https://github.com/datalad/testrepo_gh-sub__1'),
+            # verify OSF mapping, but see
+            # https://github.com/datalad/datalad/issues/5769 for future
+            # implications
+            ({},
+             'https://osf.io/q8xnk/',
+             'osf://q8xnk'),
+            ):
+        with patch.dict(cfg._merged_store, m):
+            eq_(_map_urls(cfg, [i]), [o])
